import React, { useEffect, useState } from "react";
import { VehicleTypeRadio, BookingTypeRadio, TableView } from "./component";
import { Grid, Box, TextField, Button, IconButton, Stack, Typography, CircularProgress } from "@mui/material";
import RefreshIcon from '@mui/icons-material/Refresh';
import { useTranslation } from "react-i18next";
import { checkEntry, entryByManual, openGate, closeGate, getLogs, getOpenList } from "../../utils/api";
import moment from "moment";

const In = () => {
  const { t } = useTranslation();
  const [plate, setPlate] = useState("");
  const [vehicleType, setVehicleType] = useState("L");
  const [bookingType, setBookingType] = useState("N");

  const [bookingTime, setBookingTime] = useState(null);
  const [checkinTime, setCheckinTime] = useState(null);
  const [parkingZone, setParkingZone] = useState(null);
  const [lastUpdateTime, setLastUpdateTime] = useState(null);
  const [tips, setTips] = useState(null);
  const [canManual, setCanManual] = useState(false);
  
  const [isValidateLoading, setIsValidateLoading] = useState(false);
  const [isManualLoading, setIsManualLoading] = useState(false);
  const [isButtonLoading, setIsButtonLoading] = useState(false);
  const [isRefreshLoading, setIsRefreshLoading] = useState(false);

  const [data, setData] = useState([]);

  const fetchLog = async () => {
    setIsRefreshLoading(true);
    const result = await getOpenList(moment().format("YYYY-MM-DD"));
    setData(result);
    setLastUpdateTime(moment().format("YYYY-MM-DD THH:mm:ss"));
    setIsRefreshLoading(false);
  };

  useEffect(() => {
    fetchLog();
  }, []);

  useEffect(() => {
    if (isValidateLoading || isManualLoading) {
      setIsButtonLoading(true);
    } else {
      setIsButtonLoading(false);
    }
  }, [isValidateLoading, isManualLoading])

  const validate = async () => {
    setIsValidateLoading(true);
    setBookingTime(null);
    setCheckinTime(null);
    setParkingZone(null);
    setTips(null);
    const result = await checkEntry(
      plate,
      vehicleType,
      bookingType);

    if (result) {
      fetchLog();
      setBookingTime(result.bookingTime);
      setCheckinTime(result.checkInTime);
      setParkingZone(result.parkingName);
      setTips(result.resultMsg);
    } else {
      setBookingTime();
      setCheckinTime();
      setParkingZone();
      setTips(t("vehicleNotFound"));
    }
    setCanManual(true);
    setIsValidateLoading(false);
  };

  const manual = async () => {
    setIsManualLoading(true);
    setBookingTime(null);
    setCheckinTime(null);
    setParkingZone(null);
    setTips(null);
    if (plate) {
      const result = await entryByManual(
        plate,
        vehicleType,
        bookingType,
      );
      if (result) {
        fetchLog();
        setBookingTime(result.bookingTime);
        setCheckinTime(result.checkInTime);
        setParkingZone(result.parkingName);
        setTips(result.resultMsg);
      } else {
        setBookingTime();
        setCheckinTime();
        setParkingZone();
        setTips(t("vehicleNotFound"));
      }
    }
    setIsManualLoading(false);
    fetchLog();
  };

  const reset = () => {
    setPlate("");
    setVehicleType("L");
    setBookingType("N");
    setBookingTime(null);
    setCheckinTime(null);
    setParkingZone(null);
    setCanManual(false);
    setTips(null);
  };

  const callOpenGate = async () => {
    const result = await openGate();
    console.log(result);
    fetchLog();
  };

  const callCloseGate = async () => {
    const result = await closeGate();
    console.log(result);
    fetchLog();
  };

  return (
    <Grid container xs>
      <Grid item container xs direction="column">
        <Grid item xs={4}>
          <Grid
            item
            container
            xs
            sx={{
              width: "100%",
              height: "100%",
              border: "1px solid grey",
            }}
          >
            <Stack sx={{ margin: 2 }} spacing={2}>
              <TextField
                id="outlined-basic"
                label={t("inputPlate")}
                variant="outlined"
                value={plate}
                onChange={(event) => {
                  setPlate(event.target.value);
                  setCanManual(false);
                }}
              />
              <VehicleTypeRadio value={vehicleType} setValue={setVehicleType} />
              <BookingTypeRadio value={bookingType} setValue={setBookingType} />
            </Stack>
            <Stack sx={{ margin: 2 }} spacing={2}>
              <Button
                disabled={isValidateLoading || isButtonLoading}
                size="large"
                variant="contained"
                sx={{ width: 250 }}
                onClick={validate}
              >
                {isValidateLoading ? <CircularProgress color={'inherit'} size={26}/> :t("validate")}
              </Button>
              <Button
                disabled={!canManual || isButtonLoading || isManualLoading}
                size="large"
                variant="contained"
                sx={{ width: 250 }}
                onClick={manual}

              >
                {isManualLoading ? <CircularProgress color={'inherit'} size={26}/> : t("manual")}
              </Button>
              <Button
                disabled={isButtonLoading}
                size="large"
                variant="contained"
                sx={{ width: 250 }}
                onClick={reset}
              >
                {t("reset")}
              </Button>
            </Stack>
            <Stack
              sx={{
                width: "100%",
                borderTop: "1px solid grey",
                margin: 2,
                paddingTop: 2,
              }}
              direction="row"
              spacing={1}
              justifyContent="space-between"
            >
              <Typography>
                {t("bookingTime")}：{bookingTime}
              </Typography>
              <Typography>
                {t("checkinTime")}：{checkinTime}
              </Typography>
              <Typography>
                {t("parkingZone")}：<b>{parkingZone}</b>
              </Typography>
              <Box />
            </Stack>
            <Typography sx={{ margin: 2 }}>
              {t("tips")}：<b>{tips}</b>
            </Typography>
          </Grid>
        </Grid>
        <Grid item xs>
          <Box
            sx={{
              width: "100%",
              height: "100%",
              border: "1px solid grey",
            }}
          >
            <Box>
              <Stack
                direction={'row'}
                justifyContent={'flex-end'}
                alignItems={'center'}
              >
                <Typography>
                  {`${t("lastUpdate")}: ${lastUpdateTime || 'Loading...'}`}
                </Typography>
                
                <IconButton onClick={fetchLog} disabled={isRefreshLoading}>
                {!isRefreshLoading ?<RefreshIcon /> : <CircularProgress color={'inherit'} size={24}/>}
                </IconButton>
              </Stack>
            </Box>
            <TableView data={data} />
          </Box>
        </Grid>
      </Grid>
      <Grid item container xs={2}>
        <Box
          sx={{
            width: "100%",
            height: "calc(100vh - 64px)",
            border: "1px solid grey",
            alignItems: "center",
            justifyContent: "center",
          }}
        >
          <Stack spacing={2} sx={{ margin: 2 }}>
            <Button size="large" variant="contained" onClick={callOpenGate}>
              {t("openGate")}
            </Button>
            <Button size="large" variant="contained" onClick={callCloseGate}>
              {t("closeGate")}
            </Button>
          </Stack>
        </Box>
      </Grid>
    </Grid>
  );
};

export default In;
